import { LitElement, html } from "lit-element";

import hljs from 'highlight.js/lib/highlight';
import javascript from 'highlight.js/lib/languages/javascript';
hljs.registerLanguage('javascript', javascript);

import "@code-review-core/popup-menu";
import "@code-review-core/popup-menu-item";
import "@code-review-core/circle-shape";
import { style } from "./check-item.style";

export class CheckItem extends LitElement {

    static get properties() {
        return {
            title: { type: String, attribute: true },
            state: { type: String, attribute: true, reflect: true },
            open: { type: Boolean, reflect: true },
            popupOpen: { type: Boolean }
        }
    }

    static get styles() {
        return style;
    }

    constructor() {
        super();
        this.open = false;
        this.popupOpen = false;
    }

    togglePopupMenu(e) {
        e.stopPropagation();
        this.popupOpen = !this.popupOpen;
    }

    toggle() {
        this.open = !this.open;
    }

    handleClick(e, state) {
        e.stopPropagation();
        
        const stateChangedEvent = new CustomEvent("state-changed", {
            detail: state
        });

        this.dispatchEvent(stateChangedEvent);
        this.state = state.state;
        this.popupOpen = false;
        console.log("newState: ", state);
    }

    get stateCircle() {
        switch (this.state) {
            case 'done':
                return html`<circle-shape done @click="${this.togglePopupMenu}"></circle-shape>`;
            case 'warning':
                return html`<circle-shape warning @click="${this.togglePopupMenu}"></circle-shape>`;
            case 'error':
                return html`<circle-shape error @click="${this.togglePopupMenu}"></circle-shape>`;
            default:
                return html`<circle-shape @click="${this.togglePopupMenu}"></circle-shape>`;
        }
    }

    get popupmenu() {
        const states = [
            {
                state: "done",
                title: "Good"
            },
            {
                state: "warning",
                title: "Warning",
            },
            {
                state: "error",
                title: "Error",
            },
            {
                state: "none",
                title: "None"
            },
        ];   

        return html`
            <popup-menu>
                <ul>
                    ${states.map( state => html`
                        <li>
                            <popup-menu-item 
                                state="${state.state}"
                                @click="${ e => this.handleClick(e, state)}">
                                ${state.title}
                            </popup-menu-item>
                        </li>
                    `)}
                </ul>
            </popup-menu>
        `;
    }

    render() {


        return html`
            <div class="check-item">
                <div class="summary" @click="${this.toggle}">
                    <div class="left-column">
                        ${this.stateCircle}
                        ${ this.popupOpen ?
                            this.popupmenu : null
                        }
                    </div>
                    <div class="right-column">
                        <h2 class="title">${this.title}</h2>
                    </div>
                </div>
                ${this.open ? 
                    html`
                        <div class="content">
                            <slot></slot>
                        </div>
                    ` :
                    null
                }
            </div>
        `;
    }

}

customElements.define("check-item", CheckItem);